#!/bin/bash
#

# build_numpy.sh 

# The build is qualified because it depends upon LAPACK, which is qualified.
usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e19|e20|c7>:<p382> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && pwd -P`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    for qual in "${qlist[@]}"; do
	    case $qual in
        e19|e20|c7)
        	cqual=$qual
        	;;
        p382)
        	pqual=$qual
        	;;
        *)
        	ssi_die "Unrecognized qualifier $qual"
	    esac
    done
    if [[ -z "$cqual" ]]; then
	    ssi_die "Base qualifier is mandatory"
    fi
    if [[ -z "$pqual" ]]; then
	    ssi_die "A python qualifier is mandatory (e.g. p2715)."
    fi
    basequal=${cqual}:${pqual}
    extraqual=prof
}

set_compiler_flags() {
  cflags="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
}

maybe_make_distribution() {
  [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ] && \
    ups exist -j ${package} ${pkgver} -q+${fullqual//:/:+} \
    -z "${product_dir}" && \
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball "${product_dir}" \
    ${package} ${pkgver} ${fullqual} && \
    exit 0
  return $?
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
maketar=${3}

if [[ "${maketar:-tar}" != "tar" ]]; then
  echo "WARNING: build type specifier ${maketar} no longer accepted: ignoring." 1>&2
  maketar=${4}
fi

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=numpy
origpkgver=v1_18_2
pkgver=${origpkgver}
ssibuildshims_version=v2_00_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir
get_ssibuildshims

source ssi_functions --
set_qualifiers

source define_basics NO_DEBUG_SOURCE

set_compiler_flags

maybe_make_distribution

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

if [ ${OS1} = "Darwin" ]
then
   PATH=`dropit /usr/bin/get-cert $PATH`
fi

# Set up the best Cython we can find.
cythonprod=$(latest_version -t "${product_dir}" cython \
  -q+${pqual}:+${extraqual} -z "${fakedb}" -z "${product_dir}" -z "${PRODUCTS}")
if [[ -z "${cythonprod}" ]] || ! \
  { eval setup -B ${cythonprod} -z "${fakedb}" -z "${product_dir}" -z "${PRODUCTS}"; }; then
  ssi_die "unable to set up cython (${cythonprod:--q+${pqual}:+${extraqual}}) required for build."
fi

ups active

set -x

# work in a build directory, which will not be part of any distribution
python_shortdotversion=${PYTHON_VERSION#v}
python_shortdotversion=${python_shortdotversion//_/.}
python_shortdotversion=${python_shortdotversion%.*}

mkdir -p ${pkgdir}/lib/python${python_shortdotversion}/site-packages || ssi_die "failed to create ${pkgdir}/lib/python${python_shortdotversion}/site-packages"
mkdir -p ${blddir} || ssi_die "failed to create ${blddir}"

cd ${blddir} || ssi_die "Unable to cd into ${blddir}"
tar xf ${tardir}/${pkgtarname}.tar.gz || ssi_die "Failed to untar ${pkgtarname}.tar.gz"
cd ${package}-${pkgdotver} || ssi_die "Failed to cd into ${package}-${pkgdotver}"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

echo "PYTHONPATH=${PYTHONPATH}"
echo "cythonize=$(which cythonize)"
FC=false \
CXX=false \
CFLAGS="${cflags}" \
python setup.py build -j ${ncore} install --prefix ${pkgdir} || ssi_die "python setup.py install failed"
echo "Build of ${package} successful."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

#replace hardwired python path 
sed -i'' -e '1s/^.*$/#!\/usr\/bin\/env python /' ${pkgdir}/bin/f2py || ssi_die "Failed to fix shebang for f2py"

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q+${fullqual//:/:+} -z "${product_dir}:${PRODUCTS}" || \
  ssi_dir "setup failed."
echo "$package is installed at ${pkgdir}"

# Run some tests (we need to have installed first).
echo "Running tests on installed package"

# Use -n to avoid building again and use our installed one instead.
python runtests.py -n || ssi_die "${package} tests failed."

# This must be last.
maybe_make_distribution
exit $?
